@@ -22,7 +22,7 @@ import logging
 import json
 
 from pathlib import Path
-from typing import List, Optional, Tuple, Dict, Any
+from typing import List, Optional, Tuple, Dict, Any, Union
 
 import pandas as pd
 
@@ -115,8 +115,8 @@ class _SecurityIndicators:
         for result_path in repo_path.iterdir():
             _LOGGER.info(f"Considering... {result_path}")
 
-            if "security-indicators" not in result_path.name:
-                raise Exception(f"This repo is not part of security-indicators! {result_path}")
+            if "security-indicator" not in result_path.name:
+                raise Exception(f"This repo is not part of security-indicator! {result_path}")
 
             retrieved_files: Dict[str, Any] = {result_path.name: {}}
 
@@ -224,7 +224,7 @@ class SecurityIndicatorsBandit(_SecurityIndicators):
     def aggregate_security_indicator_bandit_results(
         limit_results: bool = False,
         max_ids: int = 5,
-        is_local: bool = True,
+        is_local: bool = False,
         security_indicator_bandit_repo_path: Path = Path("security-indicators"),
     ) -> List[Any]:
         """Aggregate si_bandit results from Ceph or locally using the provided path.
@@ -243,12 +243,17 @@ class SecurityIndicatorsBandit(_SecurityIndicators):
             repo_path=security_indicator_bandit_repo_path,
         )
         security_indicator_bandit_reports = [
-            document_results[document_name] for document_id, document_results in si_reports.items()
+            document_results[document_name]
+            for document_id, document_results in si_reports.items()
+            if document_name in document_results
         ]
+
+        _LOGGER.info("Number of files that can be used is: %r" % len(security_indicator_bandit_reports))
+
         return security_indicator_bandit_reports
 
     @staticmethod
-    def extract_data_from_si_bandit_metadata(si_bandit_report: Dict[str, Any]) -> Dict[str, Any]:
+    def _extract_data_from_si_bandit_metadata(si_bandit_report: Dict[str, Any]) -> Dict[str, Any]:
         """Extract data from si-bandit report metadata.
 
         :param si_bandit_report: SI bandit report provided by Thoth SI bandit analyzer.
@@ -262,21 +267,32 @@ class SecurityIndicatorsBandit(_SecurityIndicators):
             "analyzer_si_bandit": report_metadata["analyzer"],
             "analyzer_version_si_bandit": report_metadata["analyzer_version"],
             "document_id_si_bandit": report_metadata["document_id"],
-            "package_name": report_metadata["arguments"]["si-bandit"]["package_name"],
-            "package_version": report_metadata["arguments"]["si-bandit"]["package_version"],
-            "package_index": report_metadata["arguments"]["si-bandit"]["package_index"],
+            "package_name": report_metadata["arguments"]["app.py"]["package_name"],
+            "package_version": report_metadata["arguments"]["app.py"]["package_version"],
+            "package_index": report_metadata["arguments"]["app.py"]["package_index"],
         }
 
         return extracted_metadata
 
-    def create_si_bandit_metadata_dataframe(self, si_bandit_report: Dict[str, Any]) -> pd.DataFrame:
+    def create_si_bandit_metadata_dataframe(
+        self, si_bandit_report: Dict[str, Any], analyzer_version: Optional[str] = None
+    ) -> pd.DataFrame:
         """Create si-bandit report metadata dataframe.
 
         :param si_bandit_report: SI bandit report provided by Thoth SI bandit analyzer.
+        :param analyzer_version: analyzer version filter
 
         :output metadata_df: pandas.DataFrame with metadata obtained from `extract_data_from_si_bandit_metadata`.
         """
-        metadata_si_bandit = self.extract_data_from_si_bandit_metadata(si_bandit_report=si_bandit_report)
+        if analyzer_version:
+            document_id = si_bandit_report["metadata"]["document_id"]
+            version = si_bandit_report["metadata"]["analyzer_version"]
+
+            if not int("".join(version.split("."))) >= int("".join(analyzer_version.split("."))):
+                _LOGGER.info(f"Skipping SI-bandit report: {document_id} because has version: {version}")
+                return pd.DataFrame()
+
+        metadata_si_bandit = self._extract_data_from_si_bandit_metadata(si_bandit_report=si_bandit_report)
         metadata_df = pd.DataFrame([metadata_si_bandit])
 
         return metadata_df
@@ -415,19 +431,38 @@ class SecurityIndicatorsBandit(_SecurityIndicators):
         return report_summary_df
 
     def create_si_bandit_final_dataframe(
-        self, si_bandit_report: Dict[str, Any], filters_files: Optional[List[str]] = None
+        self,
+        si_bandit_report: Dict[str, Any],
+        filters_files: Optional[List[str]] = None,
+        analyzer_version: Optional[str] = None,
     ) -> pd.DataFrame:
         """Create final si-bandit dataframe.
 
         :param si_bandit_report: SI bandit report provided by Thoth SI bandit analyzer.
         :param filters_files: List of strings of files to be filtered from analysis
         e.g. filter_files = ['/tests'] where /tests is filtered in the file path.
+        :param analyzer_version: analyzer version filter
 
         :output report_summary_df: pandas.DataFrame summary for a single si bandit report.
         """
-        # Create metadata dataframe
-        metadata_df = self.create_si_bandit_metadata_dataframe(si_bandit_report=si_bandit_report)
+        if analyzer_version:
+            document_id = si_bandit_report["metadata"]["document_id"]
+            version = si_bandit_report["metadata"]["analyzer_version"]
 
+            if not int("".join(version.split("."))) >= int("".join(analyzer_version.split("."))):
+                _LOGGER.info(f"Skipping SI-bandit report: {document_id} because has version: {version}")
+                return pd.DataFrame()
+
+        # Create metadata dataframe
+        metadata_df = self.create_si_bandit_metadata_dataframe(
+            si_bandit_report=si_bandit_report, analyzer_version=analyzer_version
+        )
+
+        if analyzer_version and metadata_df.empty:
+            _LOGGER.info(f"Skipping SI-bandit report: {document_id} because has version: {version}")
+            return pd.DataFrame()
+
+        # Create metadata dataframe
         package_name = metadata_df["package_name"][0]
         package_version = metadata_df["package_version"][0]
         package_index = metadata_df["package_index"][0]
@@ -449,13 +484,17 @@ class SecurityIndicatorsBandit(_SecurityIndicators):
         return si_bandit_report_summary_df
 
     def aggregate_si_bandit_final_dataframe(
-        self, si_bandit_reports: List[Dict[str, Any]], filters_files: Optional[List[str]] = None
+        self,
+        si_bandit_reports: List[Dict[str, Any]],
+        filters_files: Optional[List[str]] = None,
+        analyzer_version: Optional[str] = None,
     ) -> pd.DataFrame:
         """Aggregate si-bandit dataframes into final dataframe.
 
         :param si_bandit_reports: list of SI bandit report provided by Thoth SI bandit analyzer.
         :param filters_files: List of strings of files to be filtered from analysis
         e.g. filter_files = ['/tests'] where /tests is filtered in the file path.
+        :param analyzer_version: analyzer version filter
 
         :output final_df: pandas.DataFrame aggregating all SI bandit reports provided.
         """
@@ -465,15 +504,18 @@ class SecurityIndicatorsBandit(_SecurityIndicators):
 
         for si_bandit_report in si_bandit_reports:
 
+            document_id = si_bandit_report["metadata"]["document_id"]
             _LOGGER.info(f"Analyzing SI-bandit report: {counter}/{total_reports}")
 
             si_bandit_report_summary_df = self.create_si_bandit_final_dataframe(
-                si_bandit_report=si_bandit_report, filters_files=filters_files
+                si_bandit_report=si_bandit_report, filters_files=filters_files, analyzer_version=analyzer_version
             )
+            if not si_bandit_report_summary_df.empty:
+                final_df = pd.concat([final_df, si_bandit_report_summary_df], axis=0)
 
-            final_df = pd.concat([final_df, si_bandit_report_summary_df], axis=0)
-
-            counter += 1
+                counter += 1
+            else:
+                _LOGGER.info(f"Skipping SI-bandit report: {document_id} because has different version")
 
         final_df.reset_index(inplace=True, drop=True)
 
@@ -482,7 +524,7 @@ class SecurityIndicatorsBandit(_SecurityIndicators):
     def create_security_indicators_scores(self, si_bandit_df: pd.DataFrame) -> pd.DataFrame:
         """Create Security Indicators (SI) scores from si bandit outputs.
 
-        :param si_bandit_df: pandas.Dataframe as given by `aggregate_si_bandit_final_dataframe`.
+        :param si_bandit_df: pandas.DataFrame as given by `aggregate_si_bandit_final_dataframe`.
 
         :output si_bandit_df: Extend `si_bandit_df` with SI scores created using all rows (aka all packages).
         """
@@ -541,7 +583,7 @@ class SecurityIndicatorsCloc:
     def aggregate_security_indicator_cloc_results(
         limit_results: bool = False,
         max_ids: int = 5,
-        is_local: bool = True,
+        is_local: bool = False,
         security_indicator_cloc_repo_path: Path = Path("security-indicators"),
     ) -> List[Any]:
         """Aggregate si_cloc results from Ceph or locally using the provided path.
@@ -561,12 +603,17 @@ class SecurityIndicatorsCloc:
             repo_path=security_indicator_cloc_repo_path,
         )
         security_indicator_cloc_reports = [
-            document_results[document_name] for document_id, document_results in si_reports.items()
+            document_results[document_name]
+            for document_id, document_results in si_reports.items()
+            if document_name in document_results
         ]
+
+        _LOGGER.info("Number of files that can be used is: %r" % len(security_indicator_cloc_reports))
+
         return security_indicator_cloc_reports
 
     @staticmethod
-    def extract_data_from_si_cloc_metadata(si_cloc_report: Dict[str, Any]) -> Dict[str, Any]:
+    def _extract_data_from_si_cloc_metadata(si_cloc_report: Dict[str, Any]) -> Dict[str, Any]:
         """Extract data from si-cloc report metadata.
 
         :param si_cloc_report: SI cloc report provided by Thoth SI cloc analyzer.
@@ -586,14 +633,25 @@ class SecurityIndicatorsCloc:
 
         return extracted_metadata
 
-    def create_si_cloc_metadata_dataframe(self, si_cloc_report: Dict[str, Any]) -> pd.DataFrame:
+    def create_si_cloc_metadata_dataframe(
+        self, si_cloc_report: Dict[str, Any], analyzer_version: Optional[str] = None
+    ) -> pd.DataFrame:
         """Create si-cloc report metadata dataframe.
 
         :param si_cloc_report: SI cloc report provided by Thoth SI cloc analyzer.
+        :param analyzer_version: analyzer version filter
 
         :output metadata_df: pandas.DataFrame with metadata obtained from `extract_data_from_si_cloc_metadata`.
         """
-        metadata_si_cloc = self.extract_data_from_si_cloc_metadata(si_cloc_report=si_cloc_report)
+        if analyzer_version:
+            document_id = si_cloc_report["metadata"]["document_id"]
+            version = si_cloc_report["metadata"]["analyzer_version"]
+
+            if not int("".join(version.split("."))) >= int("".join(analyzer_version.split("."))):
+                _LOGGER.info(f"Skipping SI-cloc report: {document_id} because has version: {version}")
+                return pd.DataFrame()
+
+        metadata_si_cloc = self._extract_data_from_si_cloc_metadata(si_cloc_report=si_cloc_report)
         metadata_df = pd.DataFrame([metadata_si_cloc])
 
         return metadata_df
@@ -621,15 +679,30 @@ class SecurityIndicatorsCloc:
 
         return report_summary_df
 
-    def create_si_cloc_final_dataframe(self, si_cloc_report: Dict[str, Any]) -> pd.DataFrame:
+    def create_si_cloc_final_dataframe(
+        self, si_cloc_report: Dict[str, Any], analyzer_version: Optional[str] = None
+    ) -> pd.DataFrame:
         """Create final si-cloc final dataframe.
 
         :param si_cloc_report: SI cloc report provided by Thoth SI cloc analyzer.
+        :param analyzer_version: analyzer version filter
 
         :output report_summary_df: pandas.DataFrame summary for a single si cloc report.
         """
+        if analyzer_version:
+            document_id = si_cloc_report["metadata"]["document_id"]
+            version = si_cloc_report["metadata"]["analyzer_version"]
+
+            if not int("".join(version.split("."))) >= int("".join(analyzer_version.split("."))):
+                _LOGGER.info(f"Skipping SI-cloc report: {document_id} because has version: {version}")
+                return pd.DataFrame()
+
         # Create metadata dataframe
-        metadata_df = self.create_si_cloc_metadata_dataframe(si_cloc_report)
+        metadata_df = self.create_si_cloc_metadata_dataframe(si_cloc_report, analyzer_version=analyzer_version)
+
+        if analyzer_version and metadata_df.empty:
+            _LOGGER.info(f"Skipping SI-cloc report: {document_id} because has version: {version}")
+            return pd.DataFrame()
 
         package_name = metadata_df["package_name"][0]
         package_version = metadata_df["package_version"][0]
@@ -648,10 +721,13 @@ class SecurityIndicatorsCloc:
 
         return report_summary_df
 
-    def aggregate_si_cloc_final_dataframes(self, si_cloc_reports: List[Dict[str, Any]]) -> pd.DataFrame:
+    def aggregate_si_cloc_final_dataframes(
+        self, si_cloc_reports: List[Dict[str, Any]], analyzer_version: Optional[str] = None
+    ) -> pd.DataFrame:
         """Aggregate si-cloc dataframes into final dataframe.
 
         :param si_cloc_reports: list of SI cloc report provided by Thoth SI cloc analyzer.
+        :param analyzer_version: analyzer version filter
 
         :output final_df: pandas.DataFrame aggregating all SI cloc reports provided.
         """
@@ -662,12 +738,19 @@ class SecurityIndicatorsCloc:
 
         for si_cloc_report in si_cloc_reports:
 
+            document_id = si_cloc_report["metadata"]["document_id"]
             _LOGGER.info(f"Analyzing SI-cloc report: {counter}/{total_reports}")
-            si_cloc_report_summary_df = self.create_si_cloc_final_dataframe(si_cloc_report=si_cloc_report)
 
-            final_df = pd.concat([final_df, si_cloc_report_summary_df], axis=0)
+            si_cloc_report_summary_df = self.create_si_cloc_final_dataframe(
+                si_cloc_report=si_cloc_report, analyzer_version=analyzer_version
+            )
 
-            counter += 1
+            if not si_cloc_report_summary_df.empty:
+                final_df = pd.concat([final_df, si_cloc_report_summary_df], axis=0)
+
+                counter += 1
+            else:
+                _LOGGER.info(f"Skipping SI-cloc report: {document_id} because has different version")
 
         final_df.reset_index(inplace=True, drop=True)
 
@@ -684,7 +767,7 @@ class SecurityIndicatorsAggregator:
     def retrieve_security_indicator_aggregated_results(
         limit_results: bool = False,
         max_ids: int = 5,
-        is_local: bool = True,
+        is_local: bool = False,
         security_indicator_aggregated_repo_path: Path = Path("security-indicators"),
     ) -> List[Any]:
         """Retrieve si_aggregated results from Ceph or locally using the provided path.
@@ -704,30 +787,46 @@ class SecurityIndicatorsAggregator:
             repo_path=security_indicator_aggregated_repo_path,
         )
         security_indicator_aggregated_reports = [
-            document_results[document_name] for document_id, document_results in si_reports.items()
+            document_results[document_name]
+            for document_id, document_results in si_reports.items()
+            if document_name in document_results
         ]
+
+        _LOGGER.info("Number of files that can be used is: %r" % len(security_indicator_aggregated_reports))
+
         return security_indicator_aggregated_reports
 
-    def create_si_aggregated_dataframe(
+    def create_si_aggregated_results(
         self,
         si_bandit_report: Dict[str, Any],
         si_cloc_report: Dict[str, Any],
         filters_files: Optional[List[str]] = None,
-    ) -> pd.DataFrame:
-        """Create dataframe with aggregated data from SI analyzers.
+        si_bandit_version: Optional[str] = None,
+        si_cloc_version: Optional[str] = None,
+        output_json: bool = False,
+    ) -> Union[pd.DataFrame, Dict[str, Any]]:
+        """Create dataframe or json with aggregated data from SI analyzers.
 
         :param si_bandit_report: SI bandit report provided by Thoth SI bandit analyzer.
         :param si_cloc_report: SI cloc report provided by Thoth SI cloc analyzer.
         :param filters_files: List of strings of files to be filtered from analysis
         e.g. filter_files = ['/tests'] where /tests is filtered in the file path.
+        :param si_bandit_version: filter for si bandit analyzer version
+        :param si_cloc_version: filter for si cloc analyzer version
+        :param output_json: if json output is required
 
         :output aggregated_df: pandas.DataFrame aggregating all SI analyzers reports provided.
+        :output aggregated_json: pandas.DataFrame aggregating all SI analyzers reports provided.
         """
         aggregated_df = pd.DataFrame()
         si_bandit_df = self.si_bandit.create_si_bandit_final_dataframe(
-            si_bandit_report=si_bandit_report, filters_files=filters_files
+            si_bandit_report=si_bandit_report, filters_files=filters_files, analyzer_version=si_bandit_version
         )
-        si_cloc_df = self.si_cloc.create_si_cloc_final_dataframe(si_cloc_report=si_cloc_report)
+        si_cloc_df = self.si_cloc.create_si_cloc_final_dataframe(
+            si_cloc_report=si_cloc_report, analyzer_version=si_cloc_version
+        )
+        if si_bandit_df.empty or si_cloc_df.empty:
+            _LOGGER.exception("One of the analyzer results is empty!")
 
         package_info = ["package_name", "package_version", "package_index"]
         si_bandit_package = set(str(v) for v in si_bandit_df[package_info].values)
@@ -749,28 +848,11 @@ class SecurityIndicatorsAggregator:
 
         aggregated_df.reset_index(inplace=True, drop=True)
 
+        if output_json:
+            aggregated_si = aggregated_df.to_json(orient="records")  # string
+
+            aggregated_json: Dict[str, Any] = json.loads(aggregated_si)[0]
+
+            return aggregated_json
+
         return aggregated_df
-
-    def create_si_aggregated_json(
-        self,
-        si_bandit_report: Dict[str, Any],
-        si_cloc_report: Dict[str, Any],
-        filters_files: Optional[List[str]] = None,
-    ) -> Dict[str, Any]:
-        """Create json with aggregated data from SI analyzers.
-
-        :param si_bandit_report: SI bandit report provided by Thoth SI bandit analyzer.
-        :param si_cloc_report: SI cloc report provided by Thoth SI cloc analyzer.
-        :param filters_files: List of strings of files to be filtered from analysis
-        e.g. filter_files = ['/tests'] where /tests is filtered in the file path.
-
-        :output: json file with aggregated SI analyzers reports provided.
-        """
-        aggregated_df = self.create_si_aggregated_dataframe(
-            si_bandit_report=si_bandit_report, si_cloc_report=si_cloc_report, filters_files=filters_files
-        )
-        aggregated_si = aggregated_df.to_json(orient="records")  # string
-
-        aggregated_json: Dict[str, Any] = json.loads(aggregated_si)[0]
-
-        return aggregated_json
